package com.meitu.wheecam.tool.privacy

import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.content.Context
import android.graphics.Color
import android.os.Handler
import android.os.Looper
import android.text.SpannableString
import android.text.Spanned
import android.text.method.LinkMovementMethod
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.animation.AccelerateDecelerateInterpolator
import android.widget.FrameLayout
import android.widget.RadioButton
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import com.meitu.wheecam.R
import com.meitu.wheecam.common.utils.UIHelper
import com.meitu.wheecam.common.widget.ArrowDownView
import com.meitu.wheecam.community.utils.sp.SharedPreferencesUtils
import com.meitu.wheecam.main.setting.AgreementURL
import com.meitu.wheecam.main.setting.PrivacyPolicyUrl
import java.util.Locale

/**
 * 首页展示隐私的弹框，仅针对中文简体用户
 * <p>
 * Created by DaveZ 2019/2/19
 */
class PrivacyView : FrameLayout {

    companion object {
        const val SHOW_PRIVACY_KEY = "show_privacy_key"
        const val TIP = "请勾选\"同意\"，开启自拍新体验"
        const val AGREE = "同意并开始使用"
        const val PROTOCOL = "前往阅读完整《用户协议》及《隐私政策》"
        const val REFUSE = "拒绝并退出"
    }

    private lateinit var mTvPrivacyTip: TextView
    private lateinit var mVArrow: ArrowDownView
    private var mRefuseListener: View.OnClickListener? = null
    private var mDismissRunnable: Runnable? = null
    private var mTipShowFlag = false

    constructor(context: Context) : super(context) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context)
    }

    private fun init(context: Context) {
        // 如果需要展示就展示
        if (isNeedShowPrivacy()) {
            visibility = View.VISIBLE
        } else {
            return
        }

        LayoutInflater.from(context).inflate(R.layout.layout_privacy, this, true)

        // 初始化控件
        val clContainer: ConstraintLayout = findViewById(R.id.cl_privacy_container)
        mTvPrivacyTip = findViewById(R.id.tv_privacy_tip)
        mVArrow = findViewById(R.id.arrow_down_view)
        val rbPrivacyAgree: RadioButton = findViewById(R.id.rb_selected_or_not)
        val tvPrivacyProtocol: TextView = findViewById(R.id.tv_privacy_protocol)
        val tvPrivacyRefuse: TextView = findViewById(R.id.tv_privacy_refuse)
        val vBottom: View = findViewById(R.id.v_privacy_bottom)

        // 设置控件文案
        rbPrivacyAgree.text = AGREE
        tvPrivacyRefuse.text = REFUSE
        mTvPrivacyTip.text = TIP

        val content = SpannableString(PROTOCOL)
        val color = ContextCompat.getColor(context, R.color.colorff6d6d)
        content.setSpan(ForegroundColorSpan(color), 6, content.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        // 设置点击《用户协议》
        content.setSpan(PrivacyClickableSpan(context, AgreementURL.get()), 6, 12, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        // 设置点击《隐私政策》
        content.setSpan(PrivacyClickableSpan(context, PrivacyPolicyUrl.getUrl()), 13, 19, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        tvPrivacyProtocol.highlightColor = Color.TRANSPARENT
        tvPrivacyProtocol.movementMethod = LinkMovementMethod.getInstance()
        tvPrivacyProtocol.text = content

        // 添加事件监听
        clContainer.setOnClickListener {
            if (!mTipShowFlag) {
                mTipShowFlag = true

                tipAppearAnim()

                mDismissRunnable = object : Runnable {
                    override fun run() {
                        tipDismissAnim()
                        mTipShowFlag = false
                        mDismissRunnable = null
                    }
                }
                UIHelper.runOnUiThreadDelay(mDismissRunnable, 1500)
            }
        }

        rbPrivacyAgree.setOnClickListener {
            rbPrivacyAgree.setButtonDrawable(R.drawable.profile_complete_choose_ticked)
            postDelayed({ removeSelf() }, 500)
        }

        tvPrivacyRefuse.setOnClickListener {
            mRefuseListener?.onClick(it)
        }

        vBottom.setOnClickListener {
            rbPrivacyAgree.performClick()
        }
    }

    /**
     * 提示显示动画
     */
    private fun tipAppearAnim() {
        val tipAnim = ObjectAnimator.ofFloat(mTvPrivacyTip, View.ALPHA, 0.6f, 1f)
        val arrowAnim = ObjectAnimator.ofFloat(mVArrow, View.ALPHA, 0.6f, 1f)
        val animatorSet = AnimatorSet()
        animatorSet.playTogether(tipAnim, arrowAnim)
        animatorSet.duration = 300
        animatorSet.interpolator = AccelerateDecelerateInterpolator()
        animatorSet.start()
    }

    /**
     * 提示消失动画
     */
    private fun tipDismissAnim() {
        val tipAnim = ObjectAnimator.ofFloat(mTvPrivacyTip, View.ALPHA, 1f, 0f)
        val arrowAnim = ObjectAnimator.ofFloat(mVArrow, View.ALPHA, 1f, 0f)
        val animatorSet = AnimatorSet()
        animatorSet.playTogether(tipAnim, arrowAnim)
        animatorSet.duration = 300
        animatorSet.interpolator = AccelerateDecelerateInterpolator()
        animatorSet.start()
    }

    /**
     * 只有使用简体中文才弹出（产品需求）
     */
    private fun isNeedShowPrivacy(): Boolean {
        if (!"zh".equals(Locale.getDefault().language) || !"CN".equals(Locale.getDefault().country)) {
            return false
        }
        return SharedPreferencesUtils.getParam(context, SHOW_PRIVACY_KEY, true) as Boolean
    }

    /**
     * 选择同意后就移除这个布局
     */
    private fun removeSelf() {
        parent?.let {
            val vp = it as ViewGroup
            vp.removeView(this)
            setWillNotShowPrivacy()
        }

        if (mDismissRunnable != null) {
            Handler(Looper.getMainLooper()).removeCallbacks(mDismissRunnable!!)
        }
    }

    /**
     * 设置标记之后不再展示隐私弹框
     */
    private fun setWillNotShowPrivacy() {
        SharedPreferencesUtils.setParam(context, SHOW_PRIVACY_KEY, false)
    }

    /**
     * 设置点击"拒绝并退出"的监听
     */
    fun setCloseAppListener(listener: View.OnClickListener) {
        mRefuseListener = listener
    }
}